<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleCamtechCC
 * @author Sergey Kulitsky
 */

class Payment_Module_Camtech_CC extends pm_sm_api
{

    function getInitialCurrencySettings()
    {
        $res = array(
            'currency_acceptance_rules' => array(
                array(
                    'rule_name'     =>
                        ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER,
                    'rule_selected' => DB_FALSE
                ),
                array(
                    'rule_name'     => THE_ONLY_ACCEPTED,
                    'rule_selected' => DB_TRUE
                ),
                array(
                    'rule_name'     => MAIN_STORE_CURRENCY,
                    'rule_selected' => DB_FALSE
                )
            ),

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
            'accepted_currencies' => array(
                array(
                    'currency_code'   => 'AUD',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'CAD',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'CHF',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'CZK',
                    'currency_status' => 'NOT_ACCEPTED'
                ),
                array(
                    'currency_code'   => 'DKK',
                    'currency_status' => 'NOT_ACCEPTED'
                ),
                array(
                    'currency_code'   => 'EUR',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'GBP',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'HKD',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'HUF',
                    'currency_status' => 'NOT_ACCEPTED'
                ),
                array(
                    'currency_code'   => 'JPY',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'NOK',
                    'currency_status' => 'NOT_ACCEPTED'
                ),
                array(
                    'currency_code'   => 'NZD',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'PLN',
                    'currency_status' => 'NOT_ACCEPTED'
                ),
                array(
                    'currency_code'   => 'SEK',
                    'currency_status' => 'NOT_ACCEPTED'
                ),
                array(
                    'currency_code'   => 'SGD',
                    'currency_status' => 'ACCEPTED'
                ),
                array(
                    'currency_code'   => 'USD',
                    'currency_status' => 'ACCEPTED'
                )
            )
        );
        return $res;
    }

    function Payment_Module_Camtech_CC()
    {
    }

    function setErrorMessage($msg)
    {
        $this -> errorMessage = $msg;
    }

    function getErrorMessage()
    {
        return $this -> errorMessage;
    }

    function loadState()
    {
        if (modApiFunc('Session', 'is_set', 'CamtechErrorMessage'))
        {
            $this -> errorMessage = modApiFunc('Session', 'get',
                                               'CamtechErrorMessage');
            modApiFunc('Session', 'un_set', 'CamtechErrorMessage');
        }
        else
        {
            $this -> errorMessage = '';
        }
    }

    function saveState()
    {
        if ($this -> errorMessage)
            modApiFunc('Session', 'set', 'CamtechErrorMessage',
                       $this -> errorMessage);
    }

    function install()
    {
        global $application;
        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $tables = Payment_Module_Camtech_CC :: getTables();
        $query = new DB_Table_Create($tables);

        $table = 'pm_camtech_settings';
        $columns = $tables[$table]['columns'];

        list($flt, $Default_Settings) = $csv_parser -> parse_file(
            dirname(__FILE__) . '/includes/default_settings.csv'
        );
        if (count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query -> addInsertValue($setting['key'],
                                         $columns['setting_key']);
                $query -> addInsertValue($setting['value'],
                                         $columns['setting_value']);
                $application -> db -> getDB_Result($query);
            }
        }

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       PaymentModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager', 'addEventHandler',
                   'RemovePersonInfoTypeEvent', 'Payment_Module_Chase_CC',
                   'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      PaymentModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        Payment_Module_Chase_CC                    2 PersonInfoType' :
     * billingInfo
     * creditCardInfo
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if ($this -> isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this -> MessageResources = & $application -> getInstance(
                'MessageResources',
                'payment-module-camtech-messages',
                'AdminZone'
            );
            $msg = $this -> MessageResources -> getMessage(
                'MODULE_PAYMENT_CAMTECH_API_MSG_REMOVE_PERSON_INFO_TYPE'
            );

            $value = NULL;
            switch($person_info_type_tag)
            {
                case 'billingInfo':
                {
                    $value = $msg;
                    break;
                }
                case 'creditCardInfo':
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_Camtech_CC :: getTables());
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array();

        $table = 'pm_camtech_settings';
        $tables[$table] = array(
            'columns' => array(
                'setting_id'    => $table.'.setting_id',
                'setting_key'   => $table.'.setting_key',
                'setting_value' => $table.'.setting_value'
            ),
            'types'   => array(
                'setting_id'    => DBQUERY_FIELD_TYPE_INT .
                                   ' NOT NULL auto_increment',
                'setting_key'   => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL',
                'setting_value' => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
            ),
            'primary' => array(
                'setting_id'
            )
        );

        global $application;
        return $application -> addTablePrefix($tables);
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc('Checkout', 'getActiveModules', 'payment');
        return array_key_exists($this -> getUid(), $active_modules);
    }

    function getUID()
    {
        include(dirname(__FILE__) . '/includes/uid.php');
        return $uid;
    }

    function getInfo()
    {
        global $application;

        $MR = & $application -> getInstance('MessageResources',
                                            'payment-module-camtech-messages',
                                            'AdminZone');

        return array(
            'GlobalUniquePaymentModuleID' => $this -> getUid(),
            'Name' => $MR -> getMessage('MODULE_NAME'),
            'Description' => $MR -> getMessage('MODULE_DESCR'),
            'StatusMessage' => ($this -> IsActive())
                                 ? '<span class="status_online">' .
                                   $MR -> getMessage('MODULE_STATUS_ACTIVE') .
                                   '</span>'
                                 : $MR -> getMessage('MODULE_STATUS_INACTIVE')
                                   . '<span class="required">*</span>',
            'PreferencesAZViewClassName' =>
                'CheckoutPaymentModuleCamtechCCInputAZ',
            'CZInputViewClassName' => 'CheckoutPaymentModuleCamtechCCInput',
            'CZOutputViewClassName' => 'CheckoutPaymentModuleCamtechCCOutput',
            'APIClassName' => __CLASS__
        );
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',
                         array('ApiClassName' => __CLASS__,
                               'settings_table_name' => 'pm_camtech_settings'));
        $settings = array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']] = $sval['setting_value'];

        return $settings;
    }

    function getModuleURL($mode)
    {
        if ($mode == CAMTECH_MODE_PRODUCTION)
            return CAMTECH_DEFAULT_PRODUCTION_GATEWAY_URL;

        return CAMTECH_DEFAULT_TEST_GATEWAY_URL;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables = $this -> getTables();
        $stable = $tables['pm_camtech_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_camtech_settings');
            $query -> addUpdateValue($stable['setting_value'], $sval);
            $query -> WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application -> db -> PrepareSQL($query);
            $application -> db -> DB_Exec();
        }
    }

    function getPaymentMode()
    {
        return 'direct';
    }

    function getAdditionalPersonInfoVariantTag($prerequisite_type)
    {
        if ($prerequisite_type == 'creditCardInfo')
            return 'default';
        else
            return false;
    }

    function gettimestamp()
    {
        $stamp = date('YmdGis') . '000000+000';
        return $stamp;
    }

    function export_PRICE_N_A($price, $currency)
    {
        $price = ($price == PRICE_N_A) ? 0.0 : $price;
        if ($currency == 'JPY')
            return '' . floor($price);

        $cent = '' . floor(100 * ($price - floor($price))) . '00';

        return '' . floor($price) . substr($cent, 0, 2);
    }

    function makeXMLTree($data) {
        $output = array();

        $parser = xml_parser_create();

        xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
        xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
        xml_parse_into_struct($parser, $data, $values, $tags);
        xml_parser_free($parser);

        $hash_stack = array();

        foreach ($values as $key => $val)
        {
            switch ($val['type'])
            {
                case 'open':
                    array_push($hash_stack, $val['tag']);
                    break;

                case 'close':
                    array_pop($hash_stack);
                    break;

                case 'complete':
                    array_push($hash_stack, $val['tag']);
                    eval("\$output['" . implode($hash_stack, "']['") .
                         "'] = \"{$val['value']}\";");
                    array_pop($hash_stack);
                    break;
            }
        }

        return $output;
    }

    function backgroundProcess()
    {
        global $application;
        loadCoreFile('aal.class.php');

        $currency_id = modApiFunc(
            'Localization',
            'whichCurrencySendOrderToPaymentShippingGatewayIn',
            ORDER_NOT_CREATED_YET, $this -> getUid()
        );
        $currency = modApiFunc('Localization', 'getCurrencyCodeById',
                               $currency_id);

        $amount = $this -> export_PRICE_N_A(modApiFunc('Checkout',
                                                       'getOrderPrice',
                                                       'TotalToPay',
                                                       $currency_id),
                                            $currency);

        $CCInfo = modApiFunc('Checkout',
                             'getDecryptedCreditCardInfoPrerequisiteData',
                             $this->getUid());

        $messageID = md5(time());

        $settings = $this -> getSettings();

        $request = '<?xml version="1.0" encoding="UTF-8"?>' .
                   '<SecurePayMessage>' .
                   '  <MessageInfo>' .
                   '    <messageID>' . $messageID .'</messageID>' .
                   '    <messageTimestamp>' . $this -> gettimestamp() . '</messageTimestamp>' .
                   '    <timeoutValue>60</timeoutValue>' .
                   '    <apiVersion>xml-4.2</apiVersion>' .
                   '  </MessageInfo>' .
                   '  <MerchantInfo>' .
                   '    <merchantID>' . urlencode($settings['CAMTECH_MERCHANT_ID']) . '</merchantID>' .
                   '    <password>' . urlencode($settings['CAMTECH_PASSWORD']) . '</password>' .
                   '  </MerchantInfo>' .
                   '  <RequestType>payment</RequestType>' .
                   '  <Payment>' .
                   '    <TxnList count="1">' .
                   '      <Txn ID="1">' .
                   '        <txnType>0</txnType>' .
                   '        <txnSource>23</txnSource>' .
                   '        <amount>' . urlencode($amount) . '</amount>' .
                   '        <currency>' . urlencode($currency) . '</currency>' .
                   '        <purchaseOrderNo>' . urlencode(modApiFunc('Checkout', 'getNextOrderId')) . '</purchaseOrderNo>' .
                   '        <CreditCardInfo>' .
                   '          <cardNumber>' . urlencode($CCInfo["CreditCardNumber"]["value"]) . '</cardNumber>' .
                   '          <expiryDate>' . urlencode($CCInfo["ExpirationMonth"]["value"]) . '/' . urlencode(_ml_substr($CCInfo["ExpirationYear"]["value"], -2)) . '</expiryDate>' .
                   '        </CreditCardInfo>' .
                   '      </Txn>' .
                   '    </TxnList>' .
                   '  </Payment>' .
                   '</SecurePayMessage>';

        // logging request
        $log = $this -> makeXMLTree($request);
        $log['SecurePayMessage']['Payment']['TxnList']['Txn']['CreditCardInfo']['cardNumber'] = 'xxx';
        $log['SecurePayMessage']['Payment']['TxnList']['Txn']['CreditCardInfo']['expiryDate'] = 'xxx';
        $this -> addRequestLog('LOG_PM_INPUT', 'Payment Module Logs',
                               getMsg('PM_CAMTECH_CC', 'MODULE_PAYMENT_TIMELINE_HEADER_REQUEST'),
                               prepareArrayDisplay($log));
        // end of logging

        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc -> setMethod('POST');
        $bnc -> setURL($this -> getModuleURL($settings['MODULE_MODE']));
        $bnc -> setPOSTtype('application/x-www-form-urlencoded');
        $headers = array(
            'Content-type' => 'text/xml',
            'Content-length' => _ml_strlen($request),
            'Connection' => 'close'
        );
        $bnc -> setAdditionalHeaders($headers);
        $bnc -> setPOSTstring($request);
        $result = $bnc -> RunRequest();

        $result = $this -> makeXMLTree($result['body']);

        // logging response
        $log = $result;
        $log['SecurePayMessage']['Payment']['TxnList']['Txn']['CreditCardInfo']['pan'] = 'xxx';
        $log['SecurePayMessage']['Payment']['TxnList']['Txn']['CreditCardInfo']['expiryDate'] = 'xxx';
        $this -> addRequestLog('LOG_PM_INPUT', 'Payment Module Logs',
                               getMsg('PM_CAMTECH_CC', 'MODULE_PAYMENT_TIMELINE_HEADER_RESPONSE'),
                               prepareArrayDisplay($log));
        // end of logging

        if (isset($result['SecurePayMessage'])
            && $result['SecurePayMessage']['MessageInfo']['messageID'] == $messageID
            && $result['SecurePayMessage']['Status']['statusCode'] == '000'
            && $result['SecurePayMessage']['Payment']['TxnList']['Txn']['approved'] == 'Yes')
        {
            // success here...
            $order_id = modApiFunc('Checkout', 'createOrderInDB');
            modApiFunc('Cart', 'removeAllFromCart');
            modApiFunc('Checkout', 'UpdatePaymentStatusInDB', $order_id, 2, print_r($result, true));
            $step_id_to_redirect_to = modApiFunc('Checkout', 'getCurrentStepID') + 1;
        }
        else
        {
            if (!isset($result['SecurePayMessage'])
                || $result['SecurePayMessage']['MessageInfo']['messageID'] != $messageID)
            {
                $this -> setErrorMessage(getMsg('PM_CAMTECH_CC', 'MODULE_PAYMENT_UNEXPECTED_RESPONSE'));
            }
            elseif ($result['SecurePayMessage']['Status']['statusCode'] != '000')
            {
                $this -> setErrorMessage(getMsg('PM_CAMTECH_CC', 'MODULE_PAYMENT_INCORRECT_STATUS', $result['SecurePayMessage']['Status']['statusCode']));
            }
            else
            {
                $this -> setErrorMessage(getMsg('PM_CAMTECH_CC', 'MODULE_PAYMENT_DECLINE_REASON', $result['SecurePayMessage']['Payment']['TxnList']['Txn']['responseText']));
            }
            $paymentModule = modApiFunc('Checkout', 'getPrerequisiteValidationResults', 'paymentModule');
            modApiFunc('Checkout', 'setPrerequisitesValidationResultsItem', 'paymentModule', '', false, 'CHECKOUT_ERR_005', array(), $paymentModule['validatedData']);
            $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', modApiFunc('Checkout', 'getCurrentStepID'));
            modApiFunc('Checkout', 'saveState');
        }

        $request = new Request();
        $request -> setView('CheckoutView');
        $request -> setAction("SetCurrStep");
        $request -> setKey('step_id', $step_id_to_redirect_to);
        $application -> redirect($request);
    }

    var $error_message;
}
?>