<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 *
 * @package PaymentModuleCheckGWCC
 * @author Ravil Garafutdinov
 */
class Payment_Module_CheckGW_CC extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    /**
     * PaymentModuleCheckGW  constructor.
     */
    function Payment_Module_CheckGW_CC()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");
        //     : gen real UID
        $info = $this->getInfo();

        $this->getSettings();
    }

    function loadState()
    {
        if(modApiFunc('Session', 'is_Set', 'CheckGWErrorMessage'))
        {
            $this->errorMessage = modApiFunc('Session', 'get', 'CheckGWErrorMessage');
        }
        else
        {
            $this->errorMessage = null;
        }
    }

    function saveState()
    {
        if ($this->errorMessage)
        {
            modApiFunc('Session', 'set', 'CheckGWErrorMessage', $this->errorMessage);
        }
    }

    function setErrorMessage($msg)
    {
        $this->errorMessage = $msg;
    }

    function getErrorMessage()
    {
        return $this->errorMessage;
    }

    function unsetErrorMessage()
    {
        $this->errorMessage = null;
        if(modApiFunc('Session', 'is_Set', 'CheckGWErrorMessage'))
        {
            modApiFunc('Session', 'un_Set', 'CheckGWErrorMessage');
        }
    }

    function getmicrotime()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");

        list($usec, $sec) = explode(" ",microtime());

        return $obj->getMessage('MODULE_PAYMENT_CHECKGW_LOG_EVENT_TIME') . (float)($usec + $sec) . " " . $obj->getMessage('MODULE_PAYMENT_CHECKGW_SECONDS');
    }

    /**
     * Installs the specified module in the system.
     *
     * The install() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Payment_Module_CheckGW_CC::getTables() instead of $this->getTables().
     */
    function install()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");

        $tables = Payment_Module_CheckGW_CC::getTables();
        $query = new DB_Table_Create($tables);

        $table = 'pm_checkgw_settings';            #the name of the filled table
        $columns = $tables[$table]['columns'];        #the array of field names of the table

        $query = new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue("MODULE_NAME", $columns['key']);
        $query->addInsertValue('s:'._ml_strlen($obj->getMessage('MODULE_NAME')).':"'.$obj->getMessage('MODULE_NAME').'";', $columns['value']);
        $application->db->getDB_Result($query);

        $query = new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue("MODULE_STORE_ID", $columns['key']);
        $query->addInsertValue('s:0:"";', $columns['value']);
        $application->db->getDB_Result($query);

        $query = new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue("MODULE_STORE_PASSWORD", $columns['key']);
        $query->addInsertValue('s:0:"";', $columns['value']);
        $application->db->getDB_Result($query);

        $query = new DB_Insert($table);
        $query->addInsertValue(5, $columns['id']);
        $query->addInsertValue("MODULE_GATEWAY_URL", $columns['key']);
        $query->addInsertValue('s:'._ml_strlen($obj->getMessage('MODULE_DEFAULT_GATEWAY_URL')).':"'.$obj->getMessage('MODULE_DEFAULT_GATEWAY_URL').'";', $columns['value']);
        $application->db->getDB_Result($query);

        $query = new DB_Insert($table);
        $query->addInsertValue(6, $columns['id']);
        $query->addInsertValue("MODULE_CLASS_CODE", $columns['key']);
        $query->addInsertValue('s:0:"";', $columns['value']);
        $application->db->getDB_Result($query);

        $query = new DB_Insert($table);
        $query->addInsertValue(7, $columns['id']);
        $query->addInsertValue("MODULE_DESCRIPTION", $columns['key']);
        $query->addInsertValue('s:0:"";', $columns['value']);
        $application->db->getDB_Result($query);

        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Payment_Module_CheckGW_CC',
               'OnRemovePersonInfoType');
    }

    /**
     * This method is invoked by Checkout module in CZ (Customer Zone). It is
     * also invoked in AZ (Admin Zone).
     * returns - Boolean - Show this module during the process of Checkout in
     * Customer Zone or not?
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getUid()
    {
        // CheckGW UID
//        return "6AFE1791-859E-4BB2-B6DC-297127CF347B";
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");

        $this->getSettings();

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $this->Settings["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleCheckGWInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleCheckGWCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleCheckGWCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }
    /**
     * Clears the Settings table.
     */
    function clearSettingsInDB()
    {
        global $application;
        $query = new DB_Delete('pm_checkgw_settings');
        $application->db->getDB_Result($query);
    }

    /**
     * Gets current module settings from Settings.
     *
     * @return array - module settings array
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_checkgw_settings'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Sets up module attributes and logs it to the database.
     *
     * @param array $Settings - module settings array.
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['pm_checkgw_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('pm_checkgw_settings');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Sets up module attributes and logs it to the database.
     *
     * @param array $Settings -  module settings array. The undefined parameter values
     * remain unchanged.
     */
    function updateSettings($Settings)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['pm_checkgw_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('pm_checkgw_settings');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }
    }

    /**
     * It is called from Checkout. It returns the cost of selected delivery
     * method. The cost should be 0 for the majority of payment methods.
     */
    function getPaymentCost($method_id)
    {
        return 0;
    }

    /**
     * Converts the value of the monetary sum to be used _out_ ASC.
     * If the price equals PRICE_N_A, then it is changed to 0.0.
     */
    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    /**
     * Prepares and returns necessary data, passed to the payment gateway.
     *
     * @todo not all data is defined
     */
    function getConfirmationData($orderId)
    {
        return;
    }

    /**
     * Processes data on updating the order status, come from the payment gateway.
     * The flag &$bStop is set by the payment module. If it is true on return,
     * then Checkout module stops the data process, come from the payment gateway.
     */
    function processData($data, $order_id)
    {
        global $application;
        $mres = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");

        //
        // The checking on the delivery to the selected place might be
        // added here
        //
        $msg = $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" . $mres->getMessage('MODULE_PAYMENT_CHECKGW_LOG_EVENT_DESCRIPTION') . "\n" . $mres->getMessage('MODULE_PAYMENT_CHECKGW_LOG_POST_DATA') . " \n " . print_r($data["_POST"], true);

        if (isset($data["_POST"]["Success"]) && $data["_POST"]["Success"] == "True"
            && isset($data["_POST"]["Severity"]) && $data["_POST"]["Severity"] == "0")
        {
	        $EventType = "ConfirmationSuccess";
	        $result = modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, 2, $msg);
       	}
       	else
       	{
	        $EventType = "ConfirmationFailure";
	        $result = modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, 3, $msg);
       	}
	    return array("EventType" => $EventType, "statusChanged" => $result);
    }

    /**
     * Uninstalls the module.
     * It deletes all module tables.
     *
     * The uninstall() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Payment_Module_CheckGW_CC::getTables() instead of $this->getTables().
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_CheckGW_CC::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /*
     * this function is needed for correct credit card info form
     * realization
     */

    function getAdditionalPersonInfoVariantTag($prerequisite_type)
    {
        if($prerequisite_type == "bankAccountInfo")
        {
            return "default";
        }
        else
        {
            return false;
        }
    }

    /*
     * processing all payments
     */

    function backgroundProcess()
    {
        global $application, $checkout;
        loadCoreFile('aal.class.php');

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");
        $bstate =    $billingInfo->getByMask('Statemenu') ? modApiFunc("Location", "getStateCode", $billingInfo->getByMask('Statemenu')) : $billingInfo->getByMask('Statetext');
        $bcountry =  $billingInfo->getByMask('Country') ? modApiFunc("Location", "getCountryCode", $billingInfo->getByMask('Country')) : $billingInfo->getByMask('Country');

        $settings = $this->getSettings();
        $refnum =  date("YmdHis").intval(microtime()*1000000);

        $bankAccountInfoPrereq = modApiFunc("Checkout", "getAdditionalPrerequisiteName", "bankAccountInfo", $this->getUid());
        $bai_values = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", $bankAccountInfoPrereq));
        $bai_values->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $currencyId = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid());
        $currencyCode = modApiFunc("Localization", "getCurrencyCodeById", $currencyId);
        $total = $this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", $currencyId));

        $fields = array(
                     "Method"             => "Debit"
                    ,"Version"            => "1.4.2"
                    ,"ResponseXML"        => "False"
// uncomment below to get into the test mode
//                    ,"Test"               => "True"
                    ,"Login"              => $settings["MODULE_STORE_ID"]
                    ,"Password"           => $settings["MODULE_STORE_PASSWORD"]
                    ,"ReferenceNumber"    => $refnum
                    ,"SECCode"            => $settings["MODULE_CLASS_CODE"]
                    ,"Descriptor"         => $settings["MODULE_DESCRIPTOR"]
                    ,"Amount"             => $total
                    ,"RoutingNumber"      => $bai_values->getByMask('BankRoutingNumber')
                    ,"AccountNumber"      => $bai_values->getByMask('BankAccountNumber')
                    ,"Name"               => $billingInfo->getByMask('Firstname') . ' ' . $billingInfo->getByMask('Lastname')
                    ,"Address1"           => $billingInfo->getByMask('Streetline1')
                    ,"Address2"           => $billingInfo->getByMask('Streetline2')
                    ,"City"               => $billingInfo->getByMask('City')
                    ,"State"              => $bstate
                    ,"Zip"                => $billingInfo->getByMask('Postcode')
                    ,"Phone"              => $billingInfo->getByMask('Phone')
                    ,"Email"              => $billingInfo->getByMask('Email')
               );

        $content = "";
        foreach ($fields as $key => $value) {
            $content .= $key . '=' . urlencode($value) . '&';
        }

        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setMethod('POST');
        $bnc->setURL($settings["MODULE_GATEWAY_URL"]);
        $bnc->setPOSTstring($content);
        $result = $bnc->RunRequest();

    //================================ logging ===============================

        $msg = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");
        $this->addRequestLog("LOG_PM_INPUT", "Payment Module Logs", $msg->getMessage("MODULE_PAYMENT_TIMELINE_HEADER_REQUEST"), $fields);

    //================================ logging ===============================

        if (isset($result["body"]))
        {
            $arrTemp = explode("\n", $result["body"]);
            foreach ($arrTemp as $key => $value)
            {
                $temp = explode('=', $value);

                if (!isset($temp[1]))
                    continue;

                $arrRlt[$temp[0]] = trim($temp[1]);
            }
        }
        else
        {
            $arrRlt["Success"] = "no_response";
        }

    //================================ logging ===============================

        $this->addRequestLog("LOG_PM_OUTPUT", "Payment Module Logs", $msg->getMessage("MODULE_PAYMENT_TIMELINE_HEADER_RESPONSE"), $arrRlt);

    //================================ logging ===============================

        $msgObj = $application->getInstance("MessageResources", "payment-module-checkgw-messages", "AdminZone");
        if (isset($arrRlt["Success"]) && $arrRlt["Success"] == "True"
            && isset($arrRlt["Severity"]) && $arrRlt["Severity"] == "0"
            /*&& isset($arrRlt["Status"]) && $arrRlt["Status"] == "Accepted"*/)
            // Successful Payment, APPROVED
        {
            //Create order
            $order_id = modApiFunc("Checkout", "createOrderInDB");
            //                .
            //                                
            modApiFunc("Cart", "removeAllFromCart");
            $this->processData(array("_POST" => $arrRlt, "_GET" => array()), $order_id);
            $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID") + 1;
        }
        else // NO RESPONSE, DECLINED or FRAUD
        {
            if ($arrRlt["Success"] == "no_response")
	        {
	            // couldn't reach CheckGateway,
	            // looks like wrong gateway settings
	            $msg = $msgObj->getMessage("ERROR_NO_RESPONSE");
                $this->setErrorMessage($msg);
	        }
            else
            {
                $msg = $msgObj->getMessage("ERROR_SERVER_RESULT")
                    . "<br />" . "Message: " . $arrRlt["Message"];
                $this->setErrorMessage($msg);
            }
            $paymentModule = modApiFunc("Checkout", "getPrerequisiteValidationResults", "paymentModule");
            modApiFunc("Checkout", "setPrerequisitesValidationResultsItem", "paymentModule", "", false, "CHECKOUT_ERR_005", array(), $paymentModule["validatedData"]);
            $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', modApiFunc("Checkout", "getCurrentStepID"));
            modApiFunc("Checkout", "saveState");
        }

        $request = new Request();
        $request->setView  ('CheckoutView');
        $request->setAction("SetCurrStep");
        $request->setKey   ( 'step_id', $step_id_to_redirect_to);
        $application->redirect($request);
    }

    /*
     * this function checks whether the SSL works on
     * this server
     */

    function checkRequirements()
    {
        $return=array();

        // check SSL
        loadCoreFile('bouncer.php');
        $bssl = new Bouncer();
        if(!$bssl->isSSLavailable())
            $return[] = "ERROR_SSL_NOT_AVAIL";

        return $return;
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'pm_checkgw_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.pm_checkgw_setting_id'
               ,'key'               => $settings.'.pm_checkgw_setting_key'
               ,'value'             => $settings.'.pm_checkgw_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        global $application;
        return $application->addTablePrefix($tables);
    }

    /**#@-*/

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor                
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (  
     *            ), ShippingMethodInfo (              ).
     *      PaymentModule'                                                 ,
     *                                    .                                  
     *                   ,                 PersonInfoType          .      
     *                                                                  
     * PersonInfoType.
     *
     *        YourPay                        PersonInfoType' :
     * billingInfo
     * shippingInfo
     * creditCardInfo
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"payment-module-checkgw-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_PAYMENT_CHECKGW_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "billingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "bankAccountInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }

    function isBankAccountInfoNeeded()
    {
        return false;
    }

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * A flag, which is stored in the database. It indicates, if the specified
     * module is mapped in CZ Checkout.
     */
    var $bActive;

    var $OrderHistoryMessageTag;
    /**#@-*/

}
?>