<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleDIBSCC
 * @author Egor V. Derevyankin
 *
 */

class Payment_Module_DIBS_CC extends pm_sm_api
{

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_FALSE
                )
               ,array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
               ,array
                (
                    'rule_name'     =>  MAIN_STORE_CURRENCY
                   ,'rule_selected' =>  DB_FALSE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'AUD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CHF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CZK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'DKK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'GBP'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HKD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HUF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'JPY'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NOK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NZD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'PLN'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SEK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SGD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    function Payment_Module_DIBS_CC()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-dibs-messages", "AdminZone");
        $Settings = $this->getSettings();
        $this->OrderHistoryMessageTag = $obj->getMessage('MODULE_PAYMENT_DIBS_LOG_MODULE_ID') .
                $this->getUID() . "\n" . $Settings["MODULE_NAME"];
    }

    function install()
    {
        global $application;

        $tables = Payment_Module_DIBS_CC::getTables();
        $query = new DB_Table_Create($tables);

        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $table = 'pm_dibs_settings';
        $columns = $tables[$table]['columns'];

        list($flt,$Default_Settings) = $csv_parser->parse_file(dirname(__FILE__)."/includes/default_settings.csv");
        if(count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($setting["key"], $columns['setting_key']);
                $query->addInsertValue($setting["value"], $columns['setting_value']);
                $application->db->getDB_Result($query);
            };
        };

        $table = 'pm_dibs_currency';
        $columns = $tables[$table]['columns'];

        list($flt, $Currencies) = $csv_parser->parse_file(dirname(__FILE__)."/includes/dibs_currency.csv");
        if(count($Currencies) > 0)
        {
            foreach($Currencies as $key => $currency)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($currency["currency_scode"], $columns['currency_scode']);
                $query->addInsertValue($currency["currency_dcode"], $columns['currency_dcode']);
                $application->db->getDB_Result($query);
            };
        };

        $table = 'pm_dibs_langs';
        $columns = $tables[$table]['columns'];

        list($flt, $Langs) = $csv_parser->parse_file(dirname(__FILE__)."/includes/dibs_langs.csv");
        if(count($Langs) > 0)
        {
            foreach($Langs as $key => $lang)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($lang["lang_code"], $columns['lang_code']);
                $query->addInsertValue($lang["display_code"], $columns['display_code']);
                $application->db->getDB_Result($query);
            };
        };

    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_DIBS_CC::getTables());
    }

    function getUID()
    {
//        return "D5E9B1AA-F206-44AB-BA45-16054B055246";
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-dibs-messages", "AdminZone");

        $sets = $this->getSettings();

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $sets["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleDIBSCCInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleDIBSCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleDIBSCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        };

        $tables = array ();

        $table = 'pm_dibs_settings';
        $tables[$table] = array(
            'columns' => array(
                'id'                => $table.'.setting_id'
               ,'setting_key'       => $table.'.setting_key'
               ,'setting_value'     => $table.'.setting_value'
            )
           ,'types' => array(
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'       => DBQUERY_FIELD_TYPE_CHAR50
               ,'setting_value'     => DBQUERY_FIELD_TYPE_CHAR255
            )
           ,'primary' => array(
                'id'
            )
        );

        $table = 'pm_dibs_currency';
        $tables[$table] = array(
            'columns' => array(
                'currency_id'    => $table.'.cyrrency_id'
               ,'currency_scode' => $table.'.currency_scode'
               ,'currency_dcode' => $table.'.currency_dcode'
             )
           ,'types' => array(
                'currency_id'    => DBQUERY_FIELD_TYPE_INT . ' NOT NULL auto_increment'
               ,'currency_scode' => DBQUERY_FIELD_TYPE_CHAR5 . ' NOT NULL'
               ,'currency_dcode' => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
             )
           ,'primary' => array(
                'currency_id'
             )
           ,'indexes' => array(
                'IDX_scode' => 'currency_scode'
           )
        );

        $table = 'pm_dibs_langs';
        $tables[$table] = array(
            'columns' => array(
                'lang_id'       => $table.'.lang_id'
               ,'lang_code'     => $table.'.lang_code'
               ,'display_code'  => $table.'.display_code'
             )
           ,'types' => array(
                'lang_id'       => DBQUERY_FIELD_TYPE_INT . ' NOT NULL auto_increment'
               ,'lang_code'     => DBQUERY_FIELD_TYPE_CHAR5 . ' NOT NULL'
               ,'display_code'  => DBQUERY_FIELD_TYPE_CHAR10 . ' NOT NULL'
             )
           ,'primary' => array(
                'lang_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_code' => 'lang_code'
           )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_dibs_settings'));
        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }


    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['pm_dibs_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_dibs_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function getConfirmationData($orderId)
    {
        global $application;
        loadCoreFile('aal.class.php');

        $settings = $this->getSettings();

        $request = new Request();
        $request->setView('CheckoutView');
        $request->setKey('asc_oid', $orderId);
        $request->setAction('UpdatePaymentStatus');
        $accept_cancel_url = $request->getURL("", true);

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", $orderId, $this->getUid());
        $currency = modApiFunc("Localization", "getCurrencyCodeById", $currency_id);

        $confirmationData = array
            (
                 "FormAction" => PM_DIBS_SERVICE_URL
                ,"FormMethod" => "POST"
                ,"GatewayName" => "DIBS"
                ,"DataFields" => array
                (
                    'merchant' => $settings['MERCHANT_ID']
                   ,'lang' => $settings['LANG']
                   ,'capturenow' => 'true'
                   ,'delivery1.Kundenavn' => $billingInfo->getByMask('Firstname') . ' ' . $billingInfo->getByMask('Lastname')
                   ,'orderid' => $orderId
                   ,'uniqueoid' => 'true'
                   ,'amount' => intval($this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", $orderId, $this->getUid()))) * 100)
                   ,'currency' => $this->__getCurrencyCode($currency)
                   ,'accepturl' => $accept_cancel_url
                   ,'cancelurl' => $accept_cancel_url
                )
            );

        if($settings['TRNS_MODE'] == PM_DIBS_TRNS_MODE_TEST)
        {
            $confirmationData['DataFields']['test'] = 'yes';
        };

        return $confirmationData;
    }

    function processData($data, $order_id)
    {
        $result = array();

        global $application;
        $msg = &$application->getInstance('MessageResources','payment-module-dibs-messages', 'AdminZone');

        if(array_key_exists('transact',$data['_POST']))
        {
            $EventType = 'ConfirmationSuccess';
            modApiFunc("Checkout", "updateOrderPaymentProcessorOrderID", $order_id, $data['_POST']['transact']);
            $result = modApiFunc(
                'Checkout','UpdatePaymentStatusInDB',$order_id,ORDER_PAYMENT_STATUS_FULLY_PAID,
                $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" .
                $msg->getMessage('MODULE_PAYMENT_DIBS_LOG_EVENT_DESCRIPTION') .
                $msg->getMessage('MODULE_PAYMENT_DIBS_LOG_MESSAGE_COMPLETED') .
                (array_key_exists('paytype',$data['_POST']) ? ("\n" .
                    $msg->getMessage('MODULE_PAYMENT_DIBS_LOG_EXTRA')." ".
                    $msg->getMessage('MODULE_PAYMENT_DIBS_LOG_EXTRA_PAYTYPE')." ".
                    $data['_POST']['paytype']) : "")
            );
        }
        else
        {
            $EventType = 'ConfirmationFailure';
            $result = modApiFunc(
                'Checkout','UpdatePaymentStatusInDB',$order_id,ORDER_PAYMENT_STATUS_DECLINED,
                $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" .
                $msg->getMessage('MODULE_PAYMENT_DIBS_LOG_EVENT_DESCRIPTION') .
                $msg->getMessage('MODULE_PAYMENT_DIBS_LOG_MESSAGE_FAILED')
            );
        }

        return array("EventType" => $EventType, "statusChanged" => $result);
    }

    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    function getmicrotime()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-dibs-messages", "AdminZone");

        list($usec, $sec) = explode(" ", microtime());

        return $obj->getMessage('MODULE_PAYMENT_DIBS_LOG_EVENT_TIME') .
                (float)($usec + $sec) . " " .
                $obj->getMessage('MODULE_PAYMENT_DIBS_SECONDS');
    }

    function getLangsList()
    {
        global $application;
        $tables = $this->getTables();
        $langs_table = $tables['pm_dibs_langs']['columns'];

        $query = new DB_Select();
        $query->addSelectTable('pm_dibs_langs');
        $query->addSelectField('*');
        $res = $application->db->getDB_Result($query);

        return $res;
    }

    function __getCurrencyCode($currency)
    {
        global $application;
        $tables = $this->getTables();
        $currency_table = $tables['pm_dibs_currency']['columns'];

        $query = new DB_Select();
        $query->addSelectTable('pm_dibs_currency');
        $query->addSelectField($currency_table['currency_dcode'], 'dcode');
        $query->WhereValue($currency_table['currency_scode'], DB_EQ, $currency);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return 208; //default currency code; Danish crowner (DKK)
        }
        else
        {
            return intval($res[0]['dcode']);
        };
    }
};

?>