<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleEPDQCC
 * @author Egor V. Derevyankin
 *
 */

class Payment_Module_EPDQ_CC extends pm_sm_api
{

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_FALSE
                )
               ,array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
               ,array
                (
                    'rule_name'     =>  MAIN_STORE_CURRENCY
                   ,'rule_selected' =>  DB_FALSE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'AUD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CHF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CZK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'DKK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'GBP'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HKD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HUF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'JPY'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NOK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NZD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'PLN'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SEK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SGD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    function Payment_Module_EPDQ_CC()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-epdq-messages", "AdminZone");
        $Settings = $this->getSettings();
        $this->OrderHistoryMessageTag = $obj->getMessage('MODULE_PAYMENT_EPDQ_LOG_MODULE_ID') .
                $this->getUID() . "\n" .
                $Settings["MODULE_NAME"];
    }

    function install()
    {
        global $application;

        $tables = Payment_Module_EPDQ_CC::getTables();
        $query = new DB_Table_Create($tables);

        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $table = 'pm_epdq_settings';            #the name of the filled table
        $columns = $tables[$table]['columns'];        #the array of field names of the table

        list($flt,$Default_Settings) = $csv_parser->parse_file(dirname(__FILE__)."/includes/default_settings.csv");
        if(count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($setting["key"], $columns['setting_key']);
                $query->addInsertValue($setting["value"], $columns['setting_value']);
                $application->db->getDB_Result($query);
            };
        };

        $columns = $tables['pm_epdq_currency']['columns'];

        list($flt, $Currencies) = $csv_parser->parse_file(dirname(__FILE__)."/includes/currency.csv");
        if(count($Currencies) > 0)
        {
            foreach($Currencies as $key => $currency)
            {
                $query = new DB_Insert('pm_epdq_currency');
                $query->addInsertValue($currency["currency_scode"], $columns['currency_scode']);
                $query->addInsertValue($currency["currency_dcode"], $columns['currency_dcode']);
                $application->db->getDB_Result($query);
            };
        };
    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_EPDQ_CC::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    function getUID()
    {
//        return "D9E9C103-2F8D-4971-ADF5-C8F87F4BFCC6";
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-epdq-messages", "AdminZone");

        $sets = $this->getSettings();

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $sets["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleEPDQCCInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleEPDQCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleEPDQCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        };

        $tables = array ();

        $settings = 'pm_epdq_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.setting_id'
               ,'setting_key'       => $settings.'.setting_key'
               ,'setting_value'     => $settings.'.setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'       => DBQUERY_FIELD_TYPE_CHAR50
               ,'setting_value'     => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $table = 'pm_epdq_currency';
        $tables[$table] = array(
            'columns' => array(
                'currency_id'    => $table.'.cyrrency_id'
               ,'currency_scode' => $table.'.currency_scode'
               ,'currency_dcode' => $table.'.currency_dcode'
             )
           ,'types' => array(
                'currency_id'    => DBQUERY_FIELD_TYPE_INT . ' NOT NULL auto_increment'
               ,'currency_scode' => DBQUERY_FIELD_TYPE_CHAR5 . ' NOT NULL'
               ,'currency_dcode' => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
             )
           ,'primary' => array(
                'currency_id'
             )
           ,'indexes' => array(
                'IDX_scode' => 'currency_scode'
           )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_epdq_settings'));
        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['pm_epdq_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_epdq_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function getConfirmationData($orderId)
    {
        global $application;
        loadCoreFile('aal.class.php');

        $request = new Request();
        $request->setView(CURRENT_REQUEST_URL);
        $request->setAction("UpdatePaymentStatus");
        $request->setKey('asc_oid',$orderId);
        $self_link = $request->getURL("", true);

        asc_file_put_contents($application->getAppIni('PATH_CACHE_DIR').'.epdq_return_url.'.$orderId, $self_link);

        $return_url = $application->getAppIni('HTTP_URL').'avactis-system/modules/payment_module_epdq_cc/rscript/redirectCustomer.php';

        $settings = $this->getSettings();

        $epdqdata = $this->__getEncryptedEPDQData($orderId,
            number_format($this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", $orderId, $this->getUid()))), 2, '.', ''));

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $shippingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo"));
        $shippingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $billingCountry = $billingInfo->getByMask('Country')? modApiFunc("Location", "getCountryCode", $billingInfo->getByMask('Country')):"";
        $shippingCountry = $shippingInfo->getByMask('Country')? modApiFunc("Location", "getCountryCode", $shippingInfo->getByMask('Country')):"";

        $billingStateName = 'bcountyprovince';
        $billingStateValue = $billingInfo->getByMask('Statemenu')
            ? modApiFunc("Location", "getState", $billingInfo->getByMask('Statemenu'))
            : $billingInfo->getByMask('Statetext');
        if ($billingCountry == "US") {
            $billingStateName = 'bstate';
            $billingStateValue = $billingInfo->getByMask('Statemenu')
                ? modApiFunc("Location", "getStateCode", $billingInfo->getByMask('Statemenu'))
                : $billingInfo->getByMask('Statetext');
        }


        $shippingStateName = 'scountyprovince';
        $shippingStateValue = $shippingInfo->getByMask('Statemenu')
            ? modApiFunc("Location", "getState", $shippingInfo->getByMask('Statemenu'))
            : $billingInfo->getByMask('Statetext');
        if ($shippingCountry == "US") {
            $shippingStateName = 'sstate';
            $shippingStateValue = $shippingInfo->getByMask('Statemenu')
                ? modApiFunc("Location", "getStateCode", $shippingInfo->getByMask('Statemenu'))
                : $billingInfo->getByMask('Statetext');
        }

        $confirmationData = array
            (
                 "FormAction" => EPDQ_SERVICE_URL
                ,"FormMethod" => "POST"
                ,"GatewayName" => "ePDQ"
                ,"DataFields" => array
                (
                    'epdqdata' => $epdqdata
                   ,'returnurl' => $return_url
                   ,'merchantdisplayname' => $settings['MDN']
                   ,'email' => $billingInfo->getByMask('Email')
                   ,'baddr1' => $billingInfo->getByMask('Streetline1')
                   ,'baddr2' => $billingInfo->getByMask('Streetline2')
                   ,'bcity' => $billingInfo->getByMask('City')
                   ,$billingStateName => $billingStateValue
                   ,'bcountry' => $billingCountry
                   ,'bpostalcode' => $billingInfo->getByMask('Postcode')
                   ,'btelephonenumber' => $billingInfo->getByMask('Phone')
                   ,'saddr1' => $shippingInfo->getByMask('Streetline1')
                   ,'saddr2' => $shippingInfo->getByMask('Streetline2')
                   ,'scity' => $shippingInfo->getByMask('City')
                   ,$shippingStateName => $shippingStateValue
                   ,'scountry' => $shippingCountry
                   ,'spostalcode' => $shippingInfo->getByMask('Postcode')
                   ,'stelephonenumber' => $shippingInfo->getByMask('Phone')
                )
            );

        return $confirmationData;
    }

    function processData($data, $order_id)
    {

        $orderInfo = modApiFunc("Checkout", "getOrderInfo", $order_id, modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", $order_id, $this->getUid()));

        switch($orderInfo['PaymentStatusId'])
        {
            case ORDER_PAYMENT_STATUS_FULLY_PAID:
                $result = array("payment_status" => array($order_id => array("old_status" => 1, "new_status" => 2)));
                $EventType = 'ConfirmationSuccess';
                break;

            default:
                $result = array("payment_status" => array($order_id => array("old_status" => 1, "new_status" => 3)));
                $EventType = 'ConfirmationFailure';
                break;
        }

        return array("EventType" => $EventType, "statusChanged" => $result);
    }

    /**
     * function is called when a notification from ePDQ is received
     *
     * @param array $data
     * @param int $order_id
     */
    function processNotification($data, $order_id)
    {
        global $application;
        $msg = &$application->getInstance('MessageResources','payment-module-epdq-messages', 'AdminZone');
        $status = $data["_POST"]["transactionstatus"];

        switch(_ml_strtolower($status))
        {
            case 'success':
                modApiFunc(
                    'Checkout', 'UpdatePaymentStatusInDB', $order_id, ORDER_PAYMENT_STATUS_FULLY_PAID,
                    $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" .
                    $msg->getMessage('MODULE_PAYMENT_EPDQ_LOG_EVENT_DESCRIPTION') .
                    $msg->getMessage('MODULE_PAYMENT_EPDQ_LOG_MESSAGE_COMPLETED') . "\n" .
                    $msg->getMessage('MODULE_PAYMENT_EPDQ_LOG_EXTRA').print_r($data["_POST"], true)
                );
                break;

            default:
                modApiFunc(
                    'Checkout', 'UpdatePaymentStatusInDB', $order_id, ORDER_PAYMENT_STATUS_DECLINED,
                    $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" .
                    $msg->getMessage('MODULE_PAYMENT_EPDQ_LOG_EVENT_DESCRIPTION') .
                    $msg->getMessage('MODULE_PAYMENT_EPDQ_LOG_MESSAGE_FAILED') . "\n" .
                    $msg->getMessage('MODULE_PAYMENT_EPDQ_LOG_EXTRA').print_r($data["_POST"], true)
                );
                break;
        }
        return;
    }

    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    function getOrderIDFromPost()
    {
        $request = new Request();
        return $request->getValueByKey('oid');
    }

    function getmicrotime()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-epdq-messages", "AdminZone");

        list($usec, $sec) = explode(" ", microtime());

        return $obj->getMessage('MODULE_PAYMENT_EPDQ_LOG_EVENT_TIME') .
                (float)($usec + $sec) . " " .
                $obj->getMessage('MODULE_PAYMENT_EPDQ_SECONDS');
    }

    function __getEncryptedEPDQData($order_id, $order_total)
    {
        $settings = $this->getSettings();
        $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", $order_id, $this->getUid());
        $currency = modApiFunc("Localization", "getCurrencyCodeById", $currency_id);
        $post_params = array(
            'clientid'     => $settings['CLIENT_ID']
           ,'password'     => $settings['PASSWORD']
           ,'oid'          => $order_id
           ,'chargetype'   => $settings['CHARGE_TYPE'] == 'A' ? EPDQ_CHARGE_TYPE_AUTH : EPDQ_CHARGE_TYPE_PRE_AUTH
           ,'currencycode' => $this->__getCurrencyCode($currency)
           ,'total'        => $order_total
        );

        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setMethod('POST');
        $bnc->setHTTPversion('1.0');
        $bnc->setURL(EPDQ_ENC_TOOL_URL);
        $bnc->setPOSTstring($bnc->prepareDATAstring($post_params));

        $res = $bnc->RunRequest();

        $epdqdata = null;

        if($res != false)
        {
            $lines = explode("\n",$res['body']);
            foreach($lines as $line)
            {
                if(preg_match("/epdqdata/i",$line))
                {
                    preg_match("/value=\"(.+)\"/i",$line,$matches);
                    $epdqdata = $matches[1];
                };
            };
        };

        return $epdqdata;
    }

    function __getCurrencyCode($currency)
    {
        global $application;
        $tables = $this->getTables();
        $currency_table = $tables['pm_epdq_currency']['columns'];

        $query = new DB_Select();
        $query->addSelectTable('pm_epdq_currency');
        $query->addSelectField($currency_table['currency_dcode'], 'dcode');
        $query->WhereValue($currency_table['currency_scode'], DB_EQ, $currency);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return 826; //default currency code;
        }
        else
        {
            return intval($res[0]['dcode']);
        };
    }
};

?>