<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleEWayCC
 * @author Egor V. Derevyankin
 */

class Payment_Module_EWay_CC extends pm_sm_api
{

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_FALSE
                )
               ,array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
               ,array
                (
                    'rule_name'     =>  MAIN_STORE_CURRENCY
                   ,'rule_selected' =>  DB_FALSE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'AUD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CHF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CZK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'DKK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'GBP'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HKD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HUF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'JPY'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NOK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NZD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'PLN'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SEK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SGD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    function Payment_Module_EWay_CC()
    {}

    function install()
    {
        global $application;
        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $tables = Payment_Module_EWay_CC::getTables();
        $query = new DB_Table_Create($tables);

        $table = 'pm_eway_settings';
        $columns = $tables[$table]['columns'];

        list($flt,$Default_Settings) = $csv_parser->parse_file(dirname(__FILE__)."/includes/default_settings.csv");
        if(count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($setting["key"], $columns['setting_key']);
                $query->addInsertValue($setting["value"], $columns['setting_value']);
                $application->db->getDB_Result($query);
            };
        };

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       PaymentModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Payment_Module_EWay_CC',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      PaymentModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        Payment_Module_EWay_CC                    2 PersonInfoType' :
     * billingInfo
     * creditCardInfo
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"payment-module-eway-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_PAYMENT_EWAY_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "billingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "creditCardInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_EWay_CC::getTables());
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array();

        $table = 'pm_eway_settings';
        $tables[$table] = array(
            'columns' => array(
                'setting_id'    => $table.'.setting_id'
               ,'setting_key'   => $table.'.setting_key'
               ,'setting_value' => $table.'.setting_value'
             )
           ,'types'   => array(
                'setting_id'    => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'   => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'setting_value' => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'setting_id'
             )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getUID()
    {
//        return '639FFDCA-9C19-43B6-A54A-FFAAF2890F85';
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;

        $MR = &$application->getInstance('MessageResources','payment-module-eway-messages','AdminZone');

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $MR->getMessage('MODULE_NAME'),
                     "Description" => $MR->getMessage('MODULE_DESCR'),
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$MR->getMessage('MODULE_STATUS_ACTIVE')."</span>":$MR->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleEWayCCInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleEWayCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleEWayCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_eway_settings'));
        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['pm_eway_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_eway_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function getPaymentMode()
    {
        return "direct";
    }

    function getAdditionalPersonInfoVariantTag($prerequisite_type)
    {
        if($prerequisite_type == "creditCardInfo")
        {
            return "default";
        }
        else
        {
            return false;
        }
    }

    function backgroundProcess($type="direct", $transactionData = null)
    {
        global $application;
        loadCoreFile('aal.class.php');

        $settings = $this->getSettings();

        if($settings['MODULE_MODE'] == EWAY_MODE_TEST)
        {
            $eway = &$application->getInstance('EwayPayment', EWAY_DEFAULT_CUSTOMER_ID, EWAY_TEST_GATEWAY_URL);
        }
        else
        {
            $eway = &$application->getInstance('EwayPayment', $settings['MODULE_CUSTOMER_ID'], EWAY_DEFAULT_GATEWAY_URL);
        };

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $CCInfo = modApiFunc("Checkout", "getDecryptedCreditCardInfoPrerequisiteData", $this->getUid());

        //Substitute 'FirstName', 'Lastname' etc for $_POST["FieldName"] where FieldName is the name of your INPUT field on your webpage
        $eway->setCustomerFirstname($billingInfo->getByMask('Firstname'));
        $eway->setCustomerLastname($billingInfo->getByMask('Lastname'));
        $eway->setCustomerEmail($billingInfo->getByMask('Email'));
        $eway->setCardHoldersName($billingInfo->getByMask('Firstname').' '.$billingInfo->getByMask('Lastname'));
        $eway->setCardNumber($CCInfo["CreditCardNumber"]["value"]);
        $eway->setCardExpiryMonth($CCInfo["ExpirationMonth"]["value"]);
        $eway->setCardExpiryYear(_ml_substr($CCInfo["ExpirationYear"]["value"], 2, 2));
        $eway->setTotalAmount(round($this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid())) * 100)));
        $eway->setCVN($CCInfo["CreditCardVerificationNumber"]["value"]);

        $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID");
        switch ($eway->doPayment())
        {
            case EWAY_TRANSACTION_OK: // Accepted
                //Create order
                $order_id = modApiFunc("Checkout", "createOrderInDB");
                //                .
                //
                modApiFunc("Cart", "removeAllFromCart");

                $MR = &$application->getInstance('MessageResources','payment-module-eway-messages','AdminZone');

                $msg = $MR->getMessage('MODULE_PAYMENT_EWAY_LOG_MODULE_ID') . "\n" .
                    $this->getUID() . "\n" .
                    $this->getmicrotime() . "\n" .
                    $settings["MODULE_NAME"] . " ".
                    $MR->getMessage("MODULE_PAYMENT_EWAY_LOG_EVENT_DESCRIPTION") .
                    $MR->getMessage("MODULE_PAYMENT_EWAY_LOG_MESSAGE_ACCEPTED") . "\n" .

                // update order info
                modApiFunc("Checkout", "updateOrderPaymentProcessorOrderID", $order_id, $eway->getAuthCode());
                modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, ORDER_PAYMENT_STATUS_FULLY_PAID, $msg);

                $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID") + 1;
                break;
            default:
                modApiFunc('Session','set','eWay_ErrorMessage',$eway->getErrorMessage());
                $paymentModule = modApiFunc("Checkout", "getPrerequisiteValidationResults", "paymentModule");
                modApiFunc("Checkout", "setPrerequisitesValidationResultsItem", "paymentModule", "", false, "CHECKOUT_ERR_005", array(), $paymentModule["validatedData"]);
                $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', modApiFunc("Checkout", "getCurrentStepID"));
                modApiFunc("Checkout", "saveState");
                break;
        };

        $request = new Request();
        $request->setView('CheckoutView');
        $request->setAction("SetCurrStep");
        $request->setKey   ( 'step_id', $step_id_to_redirect_to);
        $application->redirect($request);

    }

    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    function getmicrotime()
    {
        global $application;
        list($usec, $sec) = explode(" ", microtime());

        $MR = &$application->getInstance('MessageResources','payment-module-eway-messages','AdminZone');

        return $MR->getMessage('MODULE_PAYMENT_EWAY_LOG_EVENT_TIME') .
                (float)($usec + $sec) . " " .
                $MR->getMessage('MODULE_PAYMENT_EWAY_SECONDS');
    }

}
?>